package mn.databank.pass.pass_mtx;

import androidx.annotation.NonNull;
import android.util.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import mn.databank.pass.pass_mtx.pass_mtx_android.PassMtxAndroid;

/** PassMtxPlugin */
public class PassMtxPlugin implements FlutterPlugin, MethodCallHandler {
  public static String TAG = "PassMtxPlugin.java";
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "pass_mtx");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    Log.i(TAG, "onMethodCall Method [" + call.method + "]");
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("getPassPing")) {
      result.success(PassMtxAndroid.PassPing());
    } else if (call.method.equals("getPassVersion")) {
      result.success(PassMtxAndroid.PassVersion());
    } else if (call.method.equals("getCard")) {
      final String user_id = call.argument("user_id");
      result.success(PassMtxAndroid.PassCardList(user_id));
    } else if (call.method.equals("deleteCard")) {
        final String user_id = call.argument("user_id");
        result.success(PassMtxAndroid.PassDeleteCard(user_id));
    } else if (call.method.equals("saveCard")) {
      final String user_id = call.argument("user_id");
      final String card_pan = call.argument("card_pan");
      final String cvv = call.argument("cvv");
      final String exp_date = call.argument("exp_date");
      final String card_holder_name = call.argument("card_holder_name");
      result.success(PassMtxAndroid.PassSaveCard(
        user_id, 
        card_pan,
        cvv,
        exp_date,
        card_holder_name
        ));
    } else if (call.method.equals("verifyCard")) {
          final String user_id = call.argument("user_id");
          final String verification_method = call.argument("verification_method");
          final String verification_value = call.argument("verification_value");
          result.success(PassMtxAndroid.PassVerifyCard(
            user_id,
            verification_method,
            verification_value));
    
    } else if (call.method.equals("passInit")) {
      final String app_id = call.argument("app_id");
      final String client_id = call.argument("client_id");
      final String private_pem = call.argument("private_pem");
      final String public_pem = call.argument("public_pem");
      result.success(PassMtxAndroid.PassInit(
        app_id,
        client_id,
        private_pem,
        public_pem
      ));
    } else if (call.method.equals("purchase")) {
        final String user_id = call.argument("user_id");
        final String order_id = call.argument("order_id");
        final String payment_token = call.argument("payment_token");
        final String payment_amount = call.argument("payment_amount");
        final String expire_date = call.argument("expire_date");
        final String db_ref_no = call.argument("db_ref_no");
        final String description = call.argument("description");
        result.success(PassMtxAndroid.PassPurchase(
          user_id,
          order_id,
          payment_token,
          expire_date,
          payment_amount,
          db_ref_no,
          description
        ));
      
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

}
